--[[
************************************************************************
Project				: Broker_AllStats
Author				: zhinjio
Project Revision	: r15
Project Date		: 20090415061404

File				: Broker_AllStats.lua
Commit Author		: zhinjio
Commit Revision		: 13
Commit Date			: 20090318055257
************************************************************************
Description	:
	Main addon file. 
TODO		:
************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "AllStats"
local FULLNAME = "Broker_AllStats"
local AllStats = LibStub( "AceAddon-3.0" ):NewAddon( MODNAME, "AceEvent-3.0" )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )
local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

_G[ MODNAME ] = AllStats
local addon = AllStats

local addonversion = "r15"
local addonauthor = "zhinjio"
local builddate = "20090415061404"

local modularOptions = {}
local tooltip
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

-- addon globals
-- addon.someValue = 0
addon.me = ""

local db
local cachedValues = {}
-- a subset of cachedValues that is just the ones we're showing in the text area
local texts = {}
local forms = {}
local shuffleIndex = 0
addon.inventoryTimer = 3

-- Here's our big table of statgroups and stats. Its used for everything from
-- auto-generating options to looping for tooltips and such, so if you're going
-- to add a stat, do it here. Format is as follows...
--	{ 
--		[ L["StatGroupName"] ] = {
--			desc	= L["TOOLTIP_GROUP_DESC"],
--			toggle	= db.options.tooltipsgroups.grouptogglevalue
--			members	= {
--				[ L["Statname"] ] = {
--					desc	= L["TEXT_STR_DESC"],
--					toggle	= db.options.textdisplay.strength,
--					func	= "custom" or functionName,
--					arg1	= funcarg1,
--					arg2	= funcarg2,
--					retarg	= index,
--			},
--		},
--	}
--					
local stats = {
	[1] = {
		name	= L["Basic Group"],
		desc	= L["TOOLTIP_BASIC_DESC"],
		toggle	= "basic",
		members = {
			[1] = {
			name	= L["Strength"],
				desc	= L["TEXT_STR_DESC"],
				toggle	= "strength",
				func	= UnitStat,
				arg1	= "player",			arg2	= 1,		retarg	= 2,
				form	= "%d",
			},
			[2] = {
				name	= L["Agility"],
				desc	= L["TEXT_AGI_DESC"],
				toggle	= "agility",
				func	= UnitStat,
				arg1	= "player",			arg2	= 2,		retarg	= 2,
				form	= "%d",
			},
			[3] = {
				name	= L["Stamina"],
				desc	= L["TEXT_STA_DESC"],
				toggle	= "stamina",
				func	= UnitStat,
				arg1	= "player",			arg2	= 3,		retarg	= 2,
				form	= "%d",
			},
			[4] = {
				name	= L["Intellect"],
				desc	= L["TEXT_INT_DESC"],
				toggle	= "intellect",
				func	= UnitStat,
				arg1	= "player",			arg2	= 4,		retarg	= 2,
				form	= "%d",
			},
			[5] = {
				name	= L["Spirit"],
				desc	= L["TEXT_SPI_DESC"],
				toggle	= "spirit",
				func	= UnitStat,
				arg1	= "player",			arg2	= 5,		retarg	= 2,
				form	= "%d",
			},
			[6] = {
				name	= L["Health"],
				desc	= L["TEXT_HEALTH_DESC"],
				toggle	= "health",
				func	= UnitHealthMax,
				arg1	= "player",			arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[7] = {
				name	= L["Power"],
				desc	= L["TEXT_POWER_DESC"],
				toggle	= "power",
				func	= UnitPowerMax,
				arg1	= "player",			arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
		},
	},
	[2] = {
		name	= L["Resistance Group"],
		desc	= L["TOOLTIP_RESIST_DESC"],
		toggle	= "resist",
		members = {
			[1] = {
				name	= L["Holy Resistance"],
				desc	= L["TEXT_HOLYRESIST_DESC"],
				toggle	= "holyresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 1,		retarg	= 2,
				form	= "%d",
			},
			[2] = {
				name	= L["Fire Resistance"],
				desc	= L["TEXT_FIRERESIST_DESC"],
				toggle	= "fireresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 2,		retarg	= 2,
				form	= "%d",
			},
			[3] = {
				name	= L["Nature Resistance"],
				desc	= L["TEXT_NATURERESIST_DESC"],
				toggle	= "natureresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 3,		retarg	= 2,
				form	= "%d",
			},
			[4] = {
				name	= L["Frost Resistance"],
				desc	= L["TEXT_FROSTRESIST_DESC"],
				toggle	= "frostresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 4,		retarg	= 2,
				form	= "%d",
			},
			[5] = {
				name	= L["Shadow Resistance"],
				desc	= L["TEXT_SHADOWRESIST_DESC"],
				toggle	= "shadowresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 5,		retarg	= 2,
				form	= "%d",
			},
			[6] = {
				name	= L["Arcane Resistance"],
				desc	= L["TEXT_ARCANERESIST_DESC"],
				toggle	= "arcaneresist",
				func	= UnitResistance,
				arg1	= "player",			arg2	= 6,		retarg	= 2,
				form	= "%d",
			},
		},
	}, 
	[3] = {
		name	= L["Defense Group"],
		desc	= L["TOOLTIP_DEFENSE_DESC"],
		toggle	= "defense",
		members	= {
			[1] = {
				name	= L["Armor"],
				desc	= L["TEXT_ARMOR_DESC"],
				toggle	= "armor",
				func	= UnitArmor,
				arg1	= "player",			arg2	= nil,		retarg	= 2,
				form	= "%d",
			},
			[2] = {
				name	= L["Defense"],
				desc	= L["TEXT_DEF_DESC"],
				toggle	= "defense",
				func	= GetCombatRating,
				arg1	= 2,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[3] = {
				name	= L["Defense Bonus"],
				desc	= L["TEXT_DEFBONUS_DESC"],
				toggle	= "defensebonus",
				func	= GetCombatRatingBonus,
				arg1	= 2,				arg2	= nil,		retarg	= 1,
				form	= "%.2f",
			},
			[4] = {
				name	= L["Defense Rating"],
				desc	= L["TEXT_DEFRATING_DESC"],
				toggle	= "defenserating",
				func	= "custom",
				form	= "%d",
			},
			[5] = {
				name	= L["Parry Rating"],
				desc	= L["TEXT_PARRYRATING_DESC"],
				toggle	= "parryrating",
				func	= GetCombatRating,
				arg1	= 4,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[6] = {
				name	= L["Parry Bonus"],
				desc	= L["TEXT_PARRYBONUS_DESC"],
				toggle	= "parrybonus",
				func	= GetCombatRatingBonus,
				arg1	= 4,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[7] = {
				name	= L["Parry Chance"],
				desc	= L["TEXT_PARRYCHANCE_DESC"],
				toggle	= "parrychance",
				func	= GetParryChance,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[8] = {
				name	= L["Dodge Rating"],
				desc	= L["TEXT_DODGERATING_DESC"],
				toggle	= "dodgerating",
				func	= GetCombatRating,
				arg1	= 3,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[9] = {
				name	= L["Dodge Bonus"],
				desc	= L["TEXT_DODGEBONUS_DESC"],
				toggle	= "dodgebonus",
				func	= GetCombatRatingBonus,
				arg1	= 3,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[10] = {
				name	= L["Dodge Chance"],
				desc	= L["TEXT_DODGECHANCE_DESC"],
				toggle	= "dodgechance",
				func	= GetDodgeChance,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[11] = {
				name	= L["Block Rating"],
				desc	= L["TEXT_BLOCKRATING_DESC"],
				toggle	= "blockrating",
				func	= GetCombatRating,
				arg1	= 5,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[12] = {
				name	= L["Block Bonus"],
				desc	= L["TEXT_BLOCKBONUS_DESC"],
				toggle	= "blockbonus",
				func	= GetCombatRatingBonus,
				arg1	= 5,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[13] = {
				name	= L["Block Percentage"],
				desc	= L["TEXT_BLOCKPERCENT_DESC"],
				toggle	= "blockpercent",
				func	= GetBlockChance,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[14] = {
				name	= L["Avoidance"],
				desc	= L["TEXT_AVOIDANCE_DESC"],
				toggle	= "avoidance",
				func	= "custom",
				form	= "%.2f%%",
			},
			[15] = {
				name	= L["Resilience"],
				desc	= L["TEXT_RESILIENCE_DESC"],
				toggle	= "resilience",
				func	= "custom",
				form	= "%s",
			},
		},
	},  
	[4] = {
		name	= L["Ranged Group"],
		desc	= L["TOOLTIP_RANGED_DESC"],
		toggle	= "ranged",
		members = {
			[1] = {
				name	= L["Ranged Speed"],
				desc	= L["TEXT_RANGESPEED_DESC"],
				toggle	= "rangespeed",
				func	= UnitRangedDamage,
				arg1	= "player",			arg2	= nil,		retarg	= 1,
				form	= "%.1f",
			},
			[2] = {
				name	= L["Ranged Damage"],
				desc	= L["TEXT_RANGEDMG_DESC"],
				toggle	= "rangedmg",
				func	= "custom",
				form	= "%s",
			},
			[3] = {
				name	= L["Ranged Attack Power"],
				desc	= L["TEXT_RANGEAP_DESC"],
				toggle	= "rangeAP",
				func	= "custom",
				form	= "%d",
			},
			[4] = {
				name	= L["Ranged Hit Rating"],
				desc	= L["TEXT_RANGEHITRATING_DESC"],
				toggle	= "rangehitrating",
				func	= GetCombatRating,
				arg1	= 7,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[5] = {
				name	= L["Ranged Hit Bonus"],
				desc	= L["TEXT_RANGEHITBONUS_DESC"],
				toggle	= "rangehitbonus",
				func	= GetCombatRatingBonus,
				arg1	= 7,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[6] = {
				name	= L["Ranged Crit Rating"],
				desc	= L["TEXT_RANGECRITRATING_DESC"],
				toggle	= "rangecritrating",
				func	= GetCombatRating,
				arg1	= 10,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[7] = {
				name	= L["Ranged Crit Bonus"],
				desc	= L["TEXT_RANGECRITBONUS_DESC"],
				toggle	= "rangecritbonus",
				func	= GetCombatRatingBonus,
				arg1	= 10,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[8] = {
				name	= L["Ranged Crit Chance"],
				desc	= L["TEXT_RANGECRITCHANCE_DESC"],
				toggle	= "rangecritchance",
				func	= GetRangedCritChance,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[9] = {
				name	= L["Ranged Haste Rating"],
				desc	= L["TEXT_RANGEHASTERATING_DESC"],
				toggle	= "rangehasterating",
				func	= GetCombatRating,
				arg1	= 19,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[10] = {
				name	= L["Ranged Haste Bonus"],
				desc	= L["TEXT_RANGEHASTEBONUS_DESC"],
				toggle	= "rangehastebonus",
				func	= GetCombatRatingBonus,
				arg1	= 19,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
		},
	}, 
	[5] = {
		name	= L["Melee Group"],
		desc	= L["TOOLTIP_MELEE_DESC"],
		toggle	= "melee",
		members	= {
			[1] = {
				name	= L["Main Hand Damage"],
				desc	= L["TEXT_MAINDMG_DESC"],
				toggle	= "maindmg",
				func	= "custom",
				form	= "%s",
			},
			[2] = {
				name	= L["Off Hand Damage"],
				desc	= L["TEXT_OFFDMG_DESC"],
				toggle	= "offdmg",
				func	= "custom",
				form	= "%s",
			},
			[3] = {
				name	= L["Main Hand Speed"],
				desc	= L["TEXT_MAINSPEED_DESC"],
				toggle	= "mainspeed",
				func	= UnitAttackSpeed,
				arg1	= "player",			arg2	= nil,		retarg	= 1,
				form	= "%.2f",
			},
			[4] = {
				name	= L["Off Hand Speed"],
				desc	= L["TEXT_OFFSPEED_DESC"],
				toggle	= "offspeed",
				func	= UnitAttackSpeed,
				arg1	= "player",			arg2	= nil,		retarg	= 2,
				form	= "%.2f",
			},
			[5] = {
				name	= L["Attack Power"],
				desc	= L["TEXT_ATTPOWER_DESC"],
				toggle	= "attpower",
				func	= "custom",
				form	= "%d",
			},
			[6] = {
				name	= L["Hit Rating"],
				desc	= L["TEXT_HITRATING_DESC"],
				toggle	= "hitrating",
				func	= GetCombatRating,
				arg1	= 6,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[7] = {
				name	= L["Hit Bonus"],
				desc	= L["TEXT_HITBONUS_DESC"],
				toggle	= "hitbonus",
				func	= GetCombatRatingBonus,
				arg1	= 6,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[8] = {
				name	= L["Crit Rating"],
				desc	= L["TEXT_CRITRATING_DESC"],
				toggle	= "critrating",
				func	= GetCombatRating,
				arg1	= 9,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[9] = {
				name	= L["Crit Bonus"],
				desc	= L["TEXT_CRITBONUS_DESC"],
				toggle	= "critbonus",
				func	= GetCombatRatingBonus,
				arg1	= 9,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[10] = {
				name	= L["Crit Chance"],
				desc	= L["TEXT_CRITCHANCE_DESC"],
				toggle	= "critchance",
				func	= GetCritChance,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[11] = {
				name	= L["Haste Rating"],
				desc	= L["TEXT_HASTERATING_DESC"],
				toggle	= "hasterating",
				func	= GetCombatRating,
				arg1	= 18,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[12] = {
				name	= L["Haste Bonus"], 
				desc	= L["TEXT_HASTEBONUS_DESC"],
				toggle	= "hastebonus",
				func	= GetCombatRatingBonus,
				arg1	= 18,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[13] = {
				name	= L["Armor Penetration"],
				desc	= L["TEXT_ARMORPEN_DESC"],
				toggle	= "armorpen",
				func	= GetArmorPenetration,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			}, 
			[14] = {
				name	= L["Expertise Rating"],
				desc	= L["TEXT_EXPERTRATING_DESC"],
				toggle	= "expertrating",
				func	= GetExpertise,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%d",
			},
			[15] = {
				name	= L["Expertise Bonus"],
				desc	= L["TEXT_EXPERTBONUS_DESC"],
				toggle	= "expertbonus",
				func	= GetExpertisePercent,
				arg1	= nil,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
			[16] = {
				name	= L["Be Parried Chance"],
				desc	= L["TEXT_BEPARRYCHANCE_DESC"],
				toggle	= "beparrychance",
				func	= GetCombatRatingBonus,
				arg1	= 24,				arg2	= nil,		retarg	= 1,
				form	= "%.2f%%",
			},
		},
	}, 
	[6] = {
		name	= L["Spell Group"],
		desc	= L["TOOLTIP_SPELL_DESC"],
		toggle	= "spell",
		members = {
			[1] = {
				name	= L["Spell Damage Bonus"],
				desc	= L["TEXT_SPELLDMG_DESC"],
				toggle	= "spelldmg",
				func	= "custom",
				form	= "%d",
			},
			[2] = {
				name	= L["Holy Damage Bonus"],
				desc	= L["TEXT_HOLYDMG_DESC"],
				toggle	= "holydmg",
				func	= GetSpellBonusDamage,
				arg1	= 2,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[3] = {
				name	= L["Fire Damage Bonus"],
				desc	= L["TEXT_FIREDMG_DESC"],
				toggle	= "firedmg",
				func	= GetSpellBonusDamage,
				arg1	= 3,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[4] = {
				name	= L["Nature Damage Bonus"],
				desc	= L["TEXT_NATUREDMG_DESC"],
				toggle	= "naturedmg",
				func	= GetSpellBonusDamage,
				arg1	= 4,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[5] = {
				name	= L["Frost Damage Bonus"],
				desc	= L["TEXT_FROSTDMG_DESC"],
				toggle	= "frostdmg",
				func	= GetSpellBonusDamage,
				arg1	= 5,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[6] = {
				name	= L["Shadow Damage Bonus"],
				desc	= L["TEXT_SHADOWDMG_DESC"],
				toggle	= "shadowdmg",
				func	= GetSpellBonusDamage,
				arg1	= 6,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[7] = {
				name	= L["Arcane Damage Bonus"],
				desc	= L["TEXT_ARCANEDMG_DESC"],
				toggle	= "arcanedmg",
				func	= GetSpellBonusDamage,
				arg1	= 7,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[8] = {
				name	= L["Healing Bonus"],
				desc	= L["TEXT_HEALING_DESC"],
				toggle	= "healing",
				func	= GetSpellBonusHealing,
				arg1	= nil,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[9] = {
				name	= L["Spell Hit Rating"],
				desc	= L["TEXT_SPELLHITRATING_DESC"],
				toggle	= "spellhitrating",
				func	= GetCombatRating,
				arg1	= 8,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[10] = {
				name	= L["Spell Hit Bonus"],
				desc	= L["TEXT_SPELLHITBONUS_DESC"],
				toggle	= "spellhitbonus",
				func	= GetCombatRatingBonus,
				arg1	= 8,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[11] = {
				name	= L["Spell Crit Rating"],
				desc	= L["TEXT_SPELLCRITRATING_DESC"],
				toggle	= "spellcritrating",
				func	= GetCombatRating,
				arg1	= 11,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[12] = {
				name	= L["Spell Crit Bonus"],
				desc	= L["TEXT_SPELLCRITBONUS_DESC"],
				toggle	= "spellcritbonus",
				func	= GetCombatRatingBonus,
				arg1	= 11,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[13] = {
				name	= L["Spell Crit Chance"],
				desc	= L["TEXT_SPELLCRIT_DESC"],
				toggle	= "spellcrit",
				func	= "custom",
				form	= "%.2f%%",
			},
			[14] = {
				name	= L["Holy Crit Chance"],
				desc	= L["TEXT_HOLYCRIT_DESC"],
				toggle	= "holycrit",
				func	= GetSpellCritChance,
				arg1	= 2,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[15] = {
				name	= L["Fire Crit Chance"],
				desc	= L["TEXT_FIRECRIT_DESC"],
				toggle	= "firecrit",
				func	= GetSpellCritChance,
				arg1	= 3,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[16] = {
				name	= L["Nature Crit Chance"],
				desc	= L["TEXT_NATURECRIT_DESC"],
				toggle	= "naturecrit",
				func	= GetSpellCritChance,
				arg1	= 4,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[17] = {
				name	= L["Frost Crit Chance"],
				desc	= L["TEXT_FROSTCRIT_DESC"],
				toggle	= "frostcrit",
				func	= GetSpellCritChance,
				arg1	= 5,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[18] = {
				name	= L["Shadow Crit Chance"],
				desc	= L["TEXT_SHADOWCRIT_DESC"],
				toggle	= "shadowcrit",
				func	= GetSpellCritChance,
				arg1	= 6,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[19] = {
				name	= L["Arcane Crit Chance"],
				desc	= L["TEXT_ARCANECRIT_DESC"],
				toggle	= "arcanecrit",
				func	= GetSpellCritChance,
				arg1	= 7,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[20] = {
				name	= L["Spell Haste Rating"],
				desc	= L["TEXT_SPELLHASTERATING_DESC"],
				toggle	= "spellhasterating",
				func	= GetCombatRating,
				arg1	= 20,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[21] = {
				name	= L["Spell Haste Bonus"],
				desc	= L["TEXT_SPELLHASTEBONUS_DESC"],
				toggle	= "spellhastebonus",
				func	= GetCombatRatingBonus,
				arg1	= 20,			arg2	= nil,			retarg	= 1,
				form	= "%.2f%%",
			},
			[22] = {
				name	= L["Spell Penetration"],
				desc	= L["TEXT_SPELLPEN_DESC"],
				toggle	= "spellpenetration",
				func	= GetSpellPenetration,
				arg1	= nil,			arg2	= nil,			retarg	= 1,
				form	= "%d",
			},
			[23] = {
				name	= L["Mana Regen"],
				desc	= L["TEXT_MANAREGEN_DESC"],
				toggle	= "manaregen",
				func	= GetManaRegen,
				arg1	= nil,			arg2	= nil,			retarg	= 1,
				form	= "%.1f",
			},
			[24] = {
				name	= L["Casting Mana Regen"],
				desc	= L["TEXT_CASTINGREGEN_DESC"],
				toggle	= "castingregen",
				func	= GetManaRegen,
				arg1	= nil,			arg2	= nil,			retarg	= 2,
				form	= "%.1f",
			},
		},
	}, 
}

function addon.Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\" .. FULLNAME .. "\\icon:" .. fh .. "\124t"
	if ( msg ~= nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " " .. MODNAME .. " : " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function addon.Debug( msg )
	if ( msg ~= nil ) then
		if ( db.options.display.debug ) then
			addon.Print( "DEBUG: " .. msg )
		end
	end
end

-- coloring agents
function Brown( text )
	return "|cffdfa050" .. text .. "|r"
end
function Yellow( text )
	return "|cffffff00" .. text .. "|r"
end
function Blue( text)
	return "|cff00ffff" .. text .. "|r"
end

local function HideTooltip()
	tooltip:SetScript( "OnLeave", nil )
	tooltip:Hide()
end

local function drawTooltip()
	tooltip:Hide()
	tooltip:Clear()

	tooltip:SetColumnLayout( 5, "LEFT", "RIGHT", "CENTER", "LEFT", "RIGHT" )
	tooltip:AddHeader( "a" )
	tooltip:SetCell( 1, 1, MODNAME, "CENTER", 5 )
	tooltip:AddLine( " " )

	-- tooltip display code here...
	local col = 1
	local colOneLineNum = 2
	local colTwoLineNum = 2
	local i = 1
	while ( stats[i] ) do
		local v = stats[i]
		if ( db.options.tooltipgroups[v.toggle] ) then
			-- Add in the heading
			local offset, row  = 0, 0
			if ( col == 1 ) then
				if ( colTwoLineNum <= colOneLineNum ) then
					tooltip:AddLine( " " )
				end
				colOneLineNum = colOneLineNum + 1
				offset = 0; row = colOneLineNum
			else
				if ( colOneLineNum <= colTwoLineNum ) then
					tooltip:AddLine( " " )
				end
				colTwoLineNum = colTwoLineNum + 1
				offset = 3; row = colTwoLineNum
			end
			tooltip:SetCell( row, ( offset + 1 ), Brown( v.name ), "LEFT" )
			addon.Debug( row.."/"..(offset + 1).."/".. v.name )
			-- Added in data lines for that heading
			local j = 1
			local savedSpellBonusDmg = 0
			local savedSpellCritChance = 0
			while ( v.members[j] ) do
				local n = v.members[j]
				local display = true
				-- Under some special conditions, we aren't going to display the value
				-- See if one of those is the case

				-- Case where we're hiding zero values and the cached value happens to be 0
				if ( ( db.options.display.hidezero == true ) and ( cachedValues[n.name] == 0 ) ) then
					display = false
				end

				-- Case where we have the nice simple generalized SpellCrit/SpellBonusDmg values
				-- being displayed, and the specific school has the same value as the general value
				if ( i == 6 ) then -- Spell Group
					local blort = cachedValues[n.name]
					if ( j == 1 ) then savedSpellBonusDmg = blort end
					if ( ( j > 1 ) and ( j < 8 ) ) then
						if ( blort == savedSpellBonusDmg ) then display = false end
					end
					if ( j == 13 ) then savedSpellCritChance = blort end
					if ( ( j > 13 ) and ( j < 20 ) ) then
						if ( blort == savedSpellCritChance ) then display = false end
					end
				end

				if ( display == true ) then
					if ( col == 1 ) then
						if ( colTwoLineNum <= colOneLineNum ) then
							tooltip:AddLine( " " )
						end
						colOneLineNum = colOneLineNum + 1
						offset = 0; row = colOneLineNum
					else
						if ( colOneLineNum <= colTwoLineNum ) then
							tooltip:AddLine( " " )
						end
						colTwoLineNum = colTwoLineNum + 1
						offset = 3; row = colTwoLineNum
					end
					tooltip:SetCell( row, ( offset + 1 ), Yellow( "  " .. n.name ), "LEFT" )
					addon.Debug( row.."/"..(offset + 1).."/".. n.name )
					tooltip:SetCell( row, ( offset + 2 ), Blue( string.format( n.form, cachedValues[n.name] ) ), "RIGHT" )
					addon.Debug( row.."/"..(offset + 2).."/".. cachedValues[n.name] )
					if ( col == 1 ) then
						tooltip:SetCell( row, 3, "   ", "CENTER" )
						addon.Debug( row.."/3/spaces" )
					end
				end
				j = j + 1
			end
			if ( col == 1 ) then col = 2 else col = 1 end
		end
		i = i + 1
	end

	tooltip:AddLine(" ")

	local lineNum = tooltip:AddLine( "b" )
	tooltip:SetCell( lineNum, 1, L["Left-Click to Refresh Values"], "LEFT", 5 )
	lineNum = tooltip:AddLine( "b" )
	tooltip:SetCell( lineNum, 1, L["Right-click for Configuration"], "LEFT", 5 )
end

-- LDB stuff
local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( MODNAME, {
	type = "data source",
	text = " ",
	label = FULLNAME,
	icon = "Interface\\Addons\\" .. FULLNAME .. "\\icon",
	OnClick = function( clickedframe, button )
				if button == "RightButton" then
					addon:ShowConfig()
				else
					addon.stopScanning()
					addon.startScanning()
					drawTooltip()
					tooltip:SmartAnchorTo( clickedframe )
					tooltip:Show()
				end
	end,
	OnEnter = function ( self )
		-- tooltip setup and scaling
		tooltip = LTT:Acquire( MODNAME.."TT", 5, "LEFT", "RIGHT", "CENTER", "LEFT", "RIGHT" )
		tooltip:SetScale( db.options.display.tipscale )
		drawTooltip()
		tooltip:EnableMouse()
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( self )
		if MouseIsOver( tooltip ) then
			tooltip:SetScript( 'OnLeave', HideTooltip )
		else
			HideTooltip()
		end
		LTT:Release( tooltip )
	end,
} )

function addon.upgradeDB()
	local olddb = addon.db.factionrealm[addon.me]
	if ( olddb.options.display.dbVersion == "1.0" ) then
		olddb.options.display.text = true
		-- tooltip groups
		olddb.options.tooltipgroups = {}
		olddb.options.tooltipgroups.basic = true
		olddb.options.tooltipgroups.resist = true
		olddb.options.tooltipgroups.defense = true
		olddb.options.tooltipgroups.melee = true
		olddb.options.tooltipgroups.ranged = true
		olddb.options.tooltipgroups.spell = true
		-- text display
		olddb.options.tooltipgroups.shuffle = true
		olddb.options.tooltipgroups.interval = 10
		olddb.options.textdisplay = {}
			-- Basic group
			olddb.options.textdisplay.strength = false
			olddb.options.textdisplay.agility = false
			olddb.options.textdisplay.stamina = false
			olddb.options.textdisplay.intellect = false
			olddb.options.textdisplay.spirit = false
			olddb.options.textdisplay.health = false
			olddb.options.textdisplay.power = false
			-- Defense Group
			olddb.options.textdisplay.armor = false
			olddb.options.textdisplay.defense = false
			olddb.options.textdisplay.defensebonus = false
			olddb.options.textdisplay.defenserating = false
			olddb.options.textdisplay.parryrating = false
			olddb.options.textdisplay.parrybonus = false
			olddb.options.textdisplay.parrychance = false
			olddb.options.textdisplay.dodgerating = false
			olddb.options.textdisplay.dodgebonus = false
			olddb.options.textdisplay.dodgechance = false
			olddb.options.textdisplay.blockrating = false
			olddb.options.textdisplay.blockbonus = false
			olddb.options.textdisplay.blockpercent = false
			olddb.options.textdisplay.avoidance = false
			-- Resistance Group
			olddb.options.textdisplay.holyresist = false
			olddb.options.textdisplay.fireresist = false
			olddb.options.textdisplay.natureresist = false
			olddb.options.textdisplay.frostresist = false
			olddb.options.textdisplay.shadowresist = false
			olddb.options.textdisplay.arcaneresist = false
			-- Melee Group
			olddb.options.textdisplay.maindmg = false
			olddb.options.textdisplay.offdmg= false
			olddb.options.textdisplay.mainspeed = false
			olddb.options.textdisplay.offspeed = false
			olddb.options.textdisplay.attpower = false
			olddb.options.textdisplay.hitrating = false
			olddb.options.textdisplay.hitbonus = false
			olddb.options.textdisplay.critrating = false
			olddb.options.textdisplay.critbonus = false
			olddb.options.textdisplay.critchance = false
			olddb.options.textdisplay.hasterating = false
			olddb.options.textdisplay.hastebonus = false
			olddb.options.textdisplay.armorpen = false
			olddb.options.textdisplay.expertrating = false
			olddb.options.textdisplay.expertbonus = false
			olddb.options.textdisplay.beparrychance = false
			-- Ranged Group
			olddb.options.textdisplay.rangespeed = false
			olddb.options.textdisplay.rangedmg = false
			olddb.options.textdisplay.rangeAP = false
			olddb.options.textdisplay.rangehitrating = false
			olddb.options.textdisplay.rangehitbonus = false
			olddb.options.textdisplay.rangecritrating = false
			olddb.options.textdisplay.rangecritbonus = false
			olddb.options.textdisplay.rangecritchance = false
			olddb.options.textdisplay.rangehasterating = false
			olddb.options.textdisplay.rangehastebonus = false
			-- Spell Group
			olddb.options.textdisplay.holydmg = false
			olddb.options.textdisplay.firedmg = false
			olddb.options.textdisplay.naturedmg = false
			olddb.options.textdisplay.frostdmg = false
			olddb.options.textdisplay.shadowdmg = false
			olddb.options.textdisplay.arcanedmg = false
			olddb.options.textdisplay.healing = false
			olddb.options.textdisplay.spellhitrating = false
			olddb.options.textdisplay.spellhitbonus = false
			olddb.options.textdisplay.spellcritrating = false
			olddb.options.textdisplay.spellcritbonus = false
			olddb.options.textdisplay.holycrit = false
			olddb.options.textdisplay.firecrit = false
			olddb.options.textdisplay.naturecrit = false
			olddb.options.textdisplay.frostcrit = false
			olddb.options.textdisplay.shadowcrit = false
			olddb.options.textdisplay.arcanecrit = false
			olddb.options.textdisplay.spellhasterating = false
			olddb.options.textdisplay.spellhastebonus = false
			olddb.options.textdisplay.spellpenetration = false
			olddb.options.textdisplay.manaregen = false
			olddb.options.textdisplay.castingregen = false
		olddb.options.display.dbVersion = "1.1"
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
	if ( olddb.options.display.dbVersion == "1.1" ) then
		olddb.options.tooltipgroups.shuffle = nil
		olddb.options.tooltipgroups.interval = nil
		olddb.options.textdisplay.shuffle = true
		olddb.options.textdisplay.interval = 10
		olddb.options.display.dbVersion = "1.2"
		addon.Print( string.format( L["Database upgraded to %s"], "1.2" ) )
	end
	if ( olddb.options.display.dbVersion == "1.2" ) then
		olddb.options.display.hidezero = false
		olddb.options.display.dbVersion = "1.3"
		addon.Print( string.format( L["Database upgraded to %s"], "1.3" ) )
	end
	if ( olddb.options.display.dbVersion == "1.3" ) then
		olddb.options.textdisplay.spelldmg = false
		olddb.options.textdisplay.spellcrit = false
		olddb.options.display.dbVersion = "1.4"
		addon.Print( string.format( L["Database upgraded to %s"], "1.4" ) )
	end
	if ( olddb.options.display.dbVersion == "1.4" ) then
		olddb.options.textdisplay.resilience = false
		olddb.options.display.dbVersion = "1.5"
		addon.Print( string.format( L["Database upgraded to %s"], "1.5" ) )
	end
end

-- options stuff
local displayOptions = nil
local tooltipOptions = nil
local textOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 2,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 21,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 22,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				blort = {
					order	= 23,
					type	= "toggle",
					name	= L["Enable DEBUG"],
					desc	= L["DISPLAY_DEBUG_DESC"],
					get		= function() return db.options.display.debug end,
					set		= function() db.options.display.debug = not db.options.display.debug end,
				},
				tooltipscale = {
					order	= 24,
					type	= "range",
					name	= L["Tooltip Scale"],
					desc	= L["TOOLTIP_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.options.display.tipscale end,
					set		= function( info, v ) 
								db.options.display.tipscale = v
								tooltip:SetScale( v )
							  end,
				},
				displaytext = {
					order	= 25,
					type	= "toggle",
					name	= L["Display Text"],
					desc	= L["DISPLAY_TEXT_DESC"],
					get		= function() return db.options.display.text end,
					set		= function() db.options.display.text = not db.options.display.text end,
				},
				hidezero = {
					order	= 26,
					type	= "toggle",
					name	= L["Hide Zero Values"],
					desc	= L["HIDE_ZERO_DESC"],
					get		= function() return db.options.display.hidezero end,
					set		= function() db.options.display.hidezero = not db.options.display.hidezero end,
				},
			},
		}
	end
	return displayOptions
end

local function giveTooltip()
	if (not tooltipOptions) then
		tooltipOptions = {
			order = 3,
			type = "group",
			name = L["Tooltip Options"],
			desc = L["TOOLTIP_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 31,
					type	= "header",
					name	= L["Tooltip Options"],
				},
				desc = {
					order	= 32,
					type	= "description",
					name	= L["TOOLTIP_OPTIONS_DESC"],
				},
			}
		}
		local i = 1
		while ( stats[i] ) do
			local v = stats[i]
			local t = {}
			t.order = i + 32
			t.type = "toggle"
			t.name = v.name
			t.desc = v.desc
			t.get = function() return db.options.tooltipgroups[v.toggle] end
			t.set = function() db.options.tooltipgroups[v.toggle] = not db.options.tooltipgroups[v.toggle] end
			local key = "data" .. ( i + 32 ) 
			tooltipOptions.args[key] = t
			i = i + 1
		end
	end
	return tooltipOptions
end

local function giveText()
	if (not textOptions) then
		textOptions = {
			order = 4,
			type = "group",
			name = L["Text Options"],
			desc = L["TEXT_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 41,
					type	= "header",
					name	= L["Text Options"],
				},
				desc = {
					order	= 42,
					type	= "description",
					name	= L["TEXT_OPTIONS_DESC"],
				},
				shuffle = {
					order	= 43,
					type	= "toggle",
					name	= L["Shuffle Values"],
					desc	= L["TEXT_SHUFFLE_DESC"],
					get		= function() return db.options.textdisplay.shuffle end,
					set		= function() db.options.textdisplay.shuffle = not db.options.textdisplay.shuffle end,
					disabled = function() return not db.options.display.text end,
				},
				interval = {
					order	= 44,
					type	= "range",
					name	= L["Shuffle Interval"],
					desc	= L["TEXT_INTERVAL_DESC"],
					min		= 5,
					max		= 30,
					step	= 1,
					bigStep = 1,
					get		= function() return db.options.textdisplay.interval end,
					set		= function( info, v ) db.options.textdisplay.interval = v end,
					disabled = function() return not ( ( db.options.display.text ) and ( db.options.textdisplay.shuffle ) ) end,
				},
			},
		}
		local i = 1
		while ( stats[i] ) do
			local v = stats[i]
			local t = {}
			t.order = ( i * 100 ) 
			t.type	= "header"
			t.name	= v.name
			local key = "header" .. ( i * 100 )
			textOptions.args[key] = t
			local j = 1
			while ( v.members[j] ) do
				local n = v.members[j]
				local u = {}
				u.order		= ( i * 100 ) + j
				u.type		= "toggle"
				u.name		= n.name
				u.desc		= n.desc
				u.get		= function() return db.options.textdisplay[n.toggle] end
				u.set		= function() db.options.textdisplay[n.toggle] = not db.options.textdisplay[n.toggle] end
				u.disabled	= function() return not db.options.display.text end
				local innerkey = "data" .. ( ( i * 100 ) + j ) 
				textOptions.args[innerkey] = u
				j = j + 1
			end
			i = i + 1
		end
	end
	return textOptions
end

local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.display.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"], " \n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
	self:RegisterModuleOptions("Tooltip", giveTooltip(), L["Tooltip Options"])
	self:RegisterModuleOptions("Text", giveText(), L["Text Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function addon:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end

function addon.updateCachedStats()
	texts = {}
	forms = {}
	local i = 1
	while ( stats[i] ) do
		local v = stats[i]
		local j = 1
		while ( v.members[j] ) do
			local n = v.members[j]
			if ( n.func ~= "custom" ) then
				cachedValues[ n.name ] = ( select( n.retarg, n.func( n.arg1, n.arg2 ) ) ) or 0
			end
			if ( db.options.textdisplay[n.toggle] ) then
				table.insert( texts, n.name )
				table.insert( forms, n.form )
			end
			j = j + 1
		end
		i = i + 1
	end
	-- Handle special cases
	local baseDefense, armorDefense				= UnitDefense( "player" )
	cachedValues[ L["Defense Rating" ] ]		= baseDefense + armorDefense
	cachedValues[ L["Avoidance"] ]				= GetDodgeChance() + GetBlockChance() + GetParryChance()
	local minDmg, maxDmg, minOffDmg, maxOffDmg	= UnitDamage( "player" )
	cachedValues[ L["Main Hand Damage"] ]		= string.format( "%.1f-%.1f", minDmg, maxDmg )
	cachedValues[ L["Off Hand Damage"] ]		= string.format( "%.1f-%.1f", minOffDmg, maxOffDmg )
	local baseAP, plusAP, minusAP				= UnitAttackPower( "player" )
	cachedValues[ L["Attack Power"] ]			= baseAP + plusAP + minusAP
	local speed, lowDmg, maxDmg = UnitRangedDamage( "player" )
	cachedValues[ L["Ranged Damage"] ]			= string.format( "%.1f-%.1f", lowDmg, maxDmg )
	local baseRAP, plusRAP, minusRAP			= UnitRangedAttackPower( "player" )
	cachedValues[ L["Ranged Attack Power"] ]	= baseRAP + plusRAP + minusRAP
	-- Spell Bonus Damage and Spell Crit Chance is divided into schools. However, these values are almost
	-- always the same. Here's an effort to make that display nicely
	local v = stats[6]
	local t = {}
	for j = 2, 8 do
		if ( v.members[j] ) then
			local n = v.members[j]
			local blort = cachedValues[n.name]
			if ( t[blort] ) then
				t[blort] = t[blort] + 1
			else
				t[blort] = 1
			end
		end
	end
	local maxval, maxindex = 0, 0
	for k, v in pairs( t ) do
		if ( v > maxval ) then maxval = v; maxindex = k end
	end
	cachedValues[ L["Spell Damage Bonus"] ] = maxindex
	t = nil; t = {}
	for j = 14, 20 do
		local n = v.members[j]; local blort = cachedValues[n.name]
		if ( t[blort] ) then t[blort] = t[blort] + 1 else t[blort] = 1 end
	end
	maxval, maxindex = 0, 0
	for k, v in pairs( t ) do
		if ( v > maxval ) then maxval = v; maxindex = k end
	end
	cachedValues[ L["Spell Crit Chance"] ] = maxindex
	-- Handle resilience
	local melee = GetCombatRating(CR_CRIT_TAKEN_MELEE)
	local ranged = GetCombatRating(CR_CRIT_TAKEN_RANGED)
	local spell = GetCombatRating(CR_CRIT_TAKEN_SPELL)
	local minResilience = min(melee, ranged)
	minResilience = min(minResilience, spell)
	cachedValues[ L["Resilience"] ] = minResilience
end

function addon.refreshText()
	launcher.text = " "
	if ( db.options.display.text ) then
		if ( next( texts ) ~= nil ) then
			local i = 1
			if ( db.options.textdisplay.shuffle ) then
				shuffleIndex = shuffleIndex + 1
				if ( shuffleIndex > #texts ) then
					shuffleIndex = 1
				end
				i = shuffleIndex
			end
			launcher.text = texts[i]..":"..string.format( forms[i], cachedValues[ texts[i] ] )
		end
	end
end

function addon.stopInventory()
	if ( AllStats_InvTimerFrame ) then
		AllStats_InvTimerFrame:SetScript( "OnUpdate", nil )
	end
end

function addon.startInventory()
	if ( not AllStats_InvTimerFrame ) then
		AllStats_InvTimerFrame = CreateFrame( "Frame" )
	end
	AllStats_InvTimerFrame:SetScript( "OnUpdate", function( self, elapsed)
		addon.inventoryTimer = addon.inventoryTimer - elapsed
		if ( addon.inventoryTimer < 0 ) then
			addon.Debug( "Have not received any inventory updates for 3 seconds. Process." )
			addon.updateCachedStats()
			addon.refreshText()
			addon.inventoryTimer = 3
			addon.stopInventory()
		end
	end )
end

function addon:ITEM_LOCK_CHANGED( ... )
	-- Reset the timer
	addon.inventoryTimer = 3
	local timerRunning = false
	if ( AllStats_InvTimerFrame ) then
		if ( AllStats_InvTimerFrame:GetScript( "OnUpdate" ) ~= nil ) then
			timerRunning = true
		end
	end
	if ( timerRunning == false ) then 
		addon.startInventory()
	end
	addon.Debug( "Received event. Resetting Timer" )
end


function addon.stopScanning()
	if ( AllStats_ScannerFrame ) then
		AllStats_ScannerFrame:SetScript( "OnUpdate", nil )
	end
end

function addon.startScanning()
	local totalElapsed = 0
	local refreshElapsed = 0

	if ( not AllStats_ScannerFrame ) then
		AllStats_ScannerFrame = CreateFrame( "Frame" )
	end

	AllStats_ScannerFrame:SetScript( "OnUpdate", function(self, elapsed)
		totalElapsed = totalElapsed + elapsed
		-- update once every 30 seconds. I may change this later
		if (totalElapsed < 2 ) then
			-- update our current cached values
			addon.updateCachedStats()
			addon.refreshText()
			totalElapsed = 2
		end
		local maxWaitTime = 30
		if ( db.options.textdisplay.shuffle ) then
			maxWaitTime = db.options.textdisplay.interval
		end
		if ( totalElapsed > maxWaitTime ) then
			totalElapsed = 0
		end
	end )
end


function addon:OnInitialize()
	-- Do Frame Creation stuff here
	-- addon:CreateFrames()
end

function addon:OnEnable()
    -- Called when the addon is loaded
	addon.me = UnitName( "player" )

	self.db = LibStub( "AceDB-3.0" ):New( MODNAME .. "_DB", nil, "Default" )
	-- do we have any data yet; for this toon? If not, init its table
	if not ( self.db.factionrealm[addon.me] ) then
		self.db.factionrealm[addon.me] = {
		-- Your saved variables stuff here
			options = {
				display = {
					debug = false,
					tipscale = 1,
					text = true,
					hidezero = false,
					dbVersion = "1.4",
				},
				tooltipgroups = {
					basic = true,
					resist = true,
					defense = true,
					melee = true,
					ranged = true,
					spell = true,
				},
				textdisplay = {					
					shuffle = true,
					interval = 10,
					-- Basic Group
					strength = false,
					agility = false,
					stamina = false,
					intellect = false,
					spirit = false,
					health = false,
					power = false,
					-- Defense Group
					armor = false,
					defense = false,
					defensebonus = false,
					defenserating = false,
					parryrating = false,
					parrybonus = false,
					parrychance = false,
					dodgerating = false,
					dodgebonus = false,
					dodgechance = false,
					blockrating = false,
					blockbonus = false,
					blockpercent = false,
					avoidance = false,
					resilience = false,
					-- Resistance Group
					holyresist = false,
					fireresist = false,
					natureresist = false,
					frostresist = false,
					shadowresist = false,
					arcaneresist = false,
					-- Melee Group
					maindmg = false,
					offdmg = false,
					mainspeed = false,
					offspeed = false,
					attpower = false,
					hitrating = false,
					hitbonus = false,
					critrating = false,
					critbonus = false,
					critchance = false,
					hasterating = false,
					hastebonus = false,
					armorpen = false,
					expertrating = false,
					expertbonus = false,
					beparrychance = false,
					-- Ranged Group
					rangespeed = false,
					rangedmg = false,
					rangeAP = false,
					rangehitrating = false,
					rangehitbonus = false,
					rangecritrating = false,
					rangecritbonus = false,
					rangecritchance = false,
					rangehasterating = false,
					rangehastebonus = false,
					-- Spell Group
					spelldmg = false,
					holydmg = false,
					firedmg = false,
					naturedmg = false,
					frostdmg = false,
					shadowdmg = false,
					arcanedmg = false,
					healing = false,
					spellhitrating = false,
					spellhitbonus = false,
					spellcritrating = false,
					spellcritbonus = false,
					spellcrit = false,
					holycrit = false,
					firecrit = false,
					naturecrit = false,
					frostcrit = false,
					shadowcrit = false,
					arcanecrit = false,
					spellhasterating = false,
					spellhastebonus = false,
					spellpenetration = false,
					manaregen = false,
					castingregen = false,
				},
			},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end
	db = self.db.factionrealm[addon.me]

	addon:SetupOptions()
	-- Event listeners
    -- addon:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	-- This event fires when someone does something in their inventory
	addon:RegisterEvent("ITEM_LOCK_CHANGED")

	-- kick off our timed scanning of stats
	addon.startScanning()
end

--[[
************************************************************************
CHANGELOG:
03/17/09 : 
	Added resilience
	Adding code to handle inventory changes
11/20/08 : 
	Initial version
************************************************************************
]]--